---
title: 'sunraster: Manipulating and Visualizing Solar Slit-spectrograph Observations in Python'
tags:
  - Python
  - astronomy
  - solar physics
  - spectroscopy
authors:
  - name: Daniel F. Ryan
    orcid: 0000-0001-8661-3825
    affiliation: "1, 2" # (Multiple affiliations must be quoted)
    corresponding: true
  - name: Nabil Freij
    orcid: 0000-0002-6253-082X
    affiliation: "3, 4"
  - name: Stuart Mumford
    orcid: 0000-0003-4217-4642
    affiliation: 5
  - name: Baptiste Pellorce
    affiliation: "6, 7"
  - name: Steven D. Christe
    orcid: 0000-0001-6127-795X
    affiliation: 8
  - name:  Ankit Kumar Baruah
    affiliation: 9
  - name: Tiago Pereira
    orcid: 0000-0003-4747-4329
    affiliation: "7, 10"
  - name: Eric Buchlin
    orcid: 0000-0003-4290-1897
    affiliation: 11
  - name: Theresa A. Kucera
    orcid: 0000-0001-9632-447X
    affiliation: 8
affiliations:
 - name: University of Applied Sciences Northwest Switzerland, Switzerland
   index: 1
 - name: American University, USA
   index: 2
 - name: Lockheed Martin Solar and Astrophysics Laboratory, USA
   index: 3
 - name: Bay Area Environmental Research Institute, USA
   index: 4
 - name: Aperio Software Ltd, UK
   index: 5
 - name: Claude Bernard Lyon 1 University, France
   index: 6
 - name: Institute of Theoretical Astrophysics, Norway
   index: 7
 - name: NASA Goddard Space Flight Center, USA
   index: 8
 - name: Workato Gmbh, Germany
   index: 9
 - name: Rosseland Centre for Solar Physics, University of Oslo, Norway
   index: 10
 - name: Université Paris-Saclay, CNRS, Institut d'Astrophysique Spatiale, France
   index: 11
date: 24 February 2023
bibliography: paper.bib

---

# Summary

sunraster is a free, open-source, community-developed Python package for inspecting,
manipulating, and visualizing solar rastering slit-spectrograph observations.
The sunraster data classes link observations with various forms of supporting data
including: measurement uncertainties; units; a data mask to mark pixels with unreliable
or un-physical data values; WCS (World Coordinate System) transformations that describe
the position, wavelengths and times represented by the pixels; and general metadata.
These classes provide support for analysis tasks, such as extracting regions of
interest and applying/removing exposure time corrections, as well as a quick-look
visualization suite.
sunraster depends heavily on the ndcube package [@ndcube-apj; @ndcube-joss] and its
SpectrogramCube class complies with the NDCube 2 API [sep12; @sep12].

# Statement of Need

Observing the location, morphology, dynamics and energetics of physical phenomena
is often key to understanding the underlying physics that drives them.
However, making simultaneous spatial, temporal and spectral measurements with a single
instrument is often challenging.
One option is to disperse the incoming light through a narrow slit onto a pixelated
detector creating a spectrogram, i.e. an image with a spectral dimension and a spatial
dimension corresponding to the long axis of the slit.
The second spatial dimension is reconstructed by scanning the slit across a region of
interest and taking sequential exposures.
A complete scan, known as a raster, creates a space-space-spectral data cube, and a time
dimension can be generated by repeating raster scans over the same region of interest.
Rastering or Scanning slit-spectrographs, as such instruments are known, have been
successfully employed for solar UV/EUV observations, e.g. the EUV Imaging
Spectrograph onboard Hinode [EIS; @eis], the Interface Region Imaging Spectrograph
[IRIS; @iris], and the SPectral Imaging for the Coronal Environment onboard Solar
Orbiter [SPICE; @spice].

However the rastering slit-spectrograph design leads to a fundamental complication,
namely that one of the spatial dimensions is coupled with time.
This can create ambiguity if the dynamic timescales of the observed phenomenon is
less than or comparable to the rastering cadence.
This requires scientists to think of their data as both 3-D (time/space-spectral-space)
and 4-D (time-spectral-space-space/time) alternately, and sometimes simultaneously,
depending on their science goals.

By building on features inherited from the ndcube package, sunraster provides an API
for seamlessly switching between the 3-D and 4-D paradigms without changing or
hard-copying the underlying data.
This saves users carrying around multiple copies of the same data which might become
inconsistent during a data analysis workflow.
A powerful slicing API enables users to extract regions of interest with single commands
that not only alter the data array, but also the associated uncertainties, data mask
and the coordinate transformations.
This saves users from performing multiple tedious and error-prone tasks on the various
components of their dataset individually.
It also makes the development of analysis workflows quicker, more reliable, and easier
to understand, thus boosting users' productivity.

# Community Applications

Most of sunraster's tools are instrument-agnostic.
However it does provide specific tools for reading Solar Orbiter/SPICE FITS files
and hence helping SPICE users leverage the wider scientific Python ecosystem
for their analysis.
In addition, the data and metadata classes in the IRIS mission's Python user
tools package [irispy-lmsal; @irispy-docs; @irispy-code] inherit from and build
upon those provided by sunraster.

# Acknowledgements

We acknowledge financial support for sunraster from Solar Orbiter/SPICE
(grant 80NSSC19K1000) as well as NASA's SDO/AIA and IRIS missions
(grant NNG04EA00C and NNG09FA40C).
We also acknowledge the SunPy and Python in Heliophysics communities for their support.

# References
